/*
History
=======
Ver. 1.0  2003-11-08  Initial release by John Land
Ver. 1.1  2005-10-27  Moved some date computation routines here from
                      SunMoonInfo.js; added several new routines;
                      cleaned up code

*/

//==============================
// Date, Day, and Week functions
//==============================

function fEasterDate(argYear) {
  // Returns an array of numbers Month (1-12), Day for Easter in argYear
  // based on the Gregorian calendar
  // based on work by E G Richards, Algorithm P - upper limit ~ AD 112000?
  if (argYear < 1583) { // Invalid Year
    return ([0, 0]);
  }

  var vA = Math.floor(argYear/100);
  var vB = vA - Math.floor(vA/4);
  var vC = argYear % 19;

  var vD = (15 + 19*vC + vB - Math.floor((vA + 1 - Math.floor((vA+8)/25))/3))%30;
  var vE = vD - Math.floor((vC+11*vD) / 319);
  var vS = 22 + vE + (140004 - (argYear + Math.floor(argYear/4))%7 + vB - vE)%7;
  // Extra %7 greatly increases year range
  var vEasterMonth = 3 + Math.floor(vS/32);
  var vEasterDay = 1 + (vS-1)%31;
  return ([vEasterMonth, vEasterDay]);
}

function fAdventDate(argYear) {
// computes closest Sunday to Nov. 30 for argYear
// returns an array of numbers for month (11=November, 12=December) and day
  with (vDate = new Date(argYear, 10, 30)) {
    setDate(getDate() - 3  + (10-getDay())%7);
    return ([getMonth()+1, getDate()]);
  }
}

function fNthWeekDay(argYear, argMonth, argWeekDay, argOccurrence) {
  // compute the Nth occurence of a weekday in a month, 
  // such as the 4th Thursday in November
  //
  // argYear       = 4-digit year
  // argMonth      = 1-12
  // argOccurrence = 1-5;
  // argOccurrence = 0 for the last occurrence of argWeekDay in the month before argMonth
  // argWeekDay    = Sun=0...Sat=6 Sun=7
  // Returns an array of numbers:
  // month (0-11)
  // day (1-31)
  with (vDate = new Date(argYear, argMonth-1, 1)){
    setDate(7*argOccurrence - 6 + (7 + argWeekDay - getDay())%7 )
  }
  return ([vDate.getMonth(), vDate.getDate()]);
}

function fDOY(argYear, argMonth, argDay) { //alternative algorithm to fDayOfYear
  // 864e5=milliseconds per day
  // Arguments: argYear, argMonth (0-11), argDay
  // Returns an array of numbers for DayOfYear, DaysToEndOfYear
  var vDateToday    = Date.UTC(argYear, argMonth, argDay);
  var vDateEOY      = Date.UTC(argYear, 11, 31);
  var vDaysToEOYNum = (vDateEOY - vDateToday)/864e5;
  var vDOYNum       = 1 + (vDateToday - Date.UTC(argYear, 0, 1))/864e5;
  return ([vDOYNum, vDaysToEOYNum]);
}

function fDayOfYear(argYear, argMonth, argDate) { //alternative algorithm to fDOY
  // Finds numerical day-of-year from year, month, and day info
  // Arguments:
  //   argYear  : 4-digit year
  //   argMonth : 1-12, January = 1
  //   argDate  : 1-31
  // Return value: The numerical day of year
  var vLeapYrAdjust = (fIsLeapYear(argYear) ? 1 : 2);
  return (Math.floor((275*argMonth)/9) - vLeapYrAdjust*Math.floor((+argMonth + 9)/12) + (+argDate) - 30);
}

function fIsLeapYear(argYear) {
  // returns 1 if 4-digit argYear is a leap year, 0 if not
  return ((argYear % 4 == 0 && argYear % 100 != 0) || argYear % 400 == 0);
}

function fDaysInMonth(argYear, argMonth) {
  // argMonth = 1-12; returns the number of days in a month, taking Leap Years into account
  var vMonth = new Date (argYear, argMonth); // get the month AFTER the month specified
  vMonth.setDate(vMonth.getDate()-1);        // subtract a day to get the last day of the desired month
  return (vMonth.getDate());
}

function fWeeksInYear(argYear) {
  // returns the number of weeks in argYear
  // an ISO8601 year may have 52 or 53 weeks, & Monday is the first day of the week
  return (52 + (new Date(argYear,00,01).getDay() == 4 || new Date(argYear,11,31).getDay() == 4));
}

function fYWDFromYMD(argYear, argMonth, argDay) {
  // 864e5=milliseconds per day
  // under ISO8601, Monday is the first day of the week
  // arguments: argYear, argMonth (0-11), argDay
  // Return values are an array of numbers for Year, Week, and DayOfWeek (1-7, 1=Monday)
  var vDateObj = new Date(argYear, argMonth, argDay);
  if (isNaN(vDateObj)) return false;

  var vDayOfWeekNum = vDateObj.getDay();
  if (vDayOfWeekNum == 0) vDayOfWeekNum = 7;

  vDateObj.setDate(vDateObj.getDate() + 4 - vDayOfWeekNum);
  var vYearNum      = vDateObj.getFullYear();
  var vZeroBasedDOY = Math.floor((vDateObj.getTime() - new Date(vYearNum,0,1,-6))/864e5);
  // Jan 1; -6h allows for Summer Time (Daylight Savings Time)

  var vWeekNum = Math.floor(vZeroBasedDOY/7) + 1;
  return ([vYearNum, vWeekNum, vDayOfWeekNum]);
}

function fJDayFromYMD(argYear, argMonth, argDay) {
  // Convert calendar date to Julian day
  // Arguments:
  //   argYear : 4 digit year
  //   argMonth: 1-12, January = 1
  //   argDay  : 1-31
  // Return value: The Julian day number corresponding to the date
  // Note: Number is returned for start of day. Fractional days should be added later.
  if (argMonth <= 2) {
    argYear  -=  1;
    argMonth += 12;
  }
  var vA = Math.floor(argYear/100);

  var vB = (argYear < 1583) ? 0 : 2 - vA + Math.floor(vA/4); // distinguish pre & post Gregorian calendar, adopted 1583
  return (Math.floor(365.25*(+argYear + 4716)) +
          Math.floor(30.6001*(+argMonth + 1)) + (+argDay) + vB - 1524.5);
}

function fYMDFromJDay(argJDay) {
  // Calendar Year, Month, and Date from Julian Day
  // Arguments: argJDay : the Julian Day to convert
  // Return values are an array of numbers for Year (4 digits), Month (1-12), Day (1-31)
  var vZ = Math.floor(argJDay + 0.5);
  var vF = argJDay + 0.5 - vZ;

  if (vZ < 2299161) {
    var vA = vZ;
  } else {
    vAlpha = Math.floor((vZ - 1867216.25)/36524.25);
    var vA = vZ + 1 + vAlpha - Math.floor(vAlpha/4);
  }

  var vB = vA + 1524;
  var vC = Math.floor((vB - 122.1)/365.25);
  var vD = Math.floor(365.25*vC);
  var vE = Math.floor((vB - vD)/30.6001);

  var vDay   = vB - vD - Math.floor(30.6001*vE) + vF;
  var vMonth = (vE < 14) ? vE-1 : vE-13;
  var vYear  = (vMonth > 2) ? vC-4716 : vC-4715;
  return ([vYear, vMonth, vDay]);
}

function fDateMonthStrFromJDay(argJDay) {
  // Calendar day (minus year) from Julian Day
  // Arguments: argJDay : the Julian Day to convert
  // Return value: String date in the form DD-MONTH
  var vYMD = fYMDFromJDay(argJDay); // returns array of YMD
  return (fLeadZero1(vYMD[2]) + ' ' + kShortMonthName[vYMD[1]-1]);
}

function fJCenturyFromJDay(argJDay) {
  // Convert Julian Day to centuries since J2000.0.
  // Arguments: argJDay : the Julian Day to convert
  // Return value: the century number corresponding to the specified Julian Day
  return ((argJDay - 2451545.0)/36525.0);
}

function fJDayFromJCentury(argJCent) {
  // Convert centuries since J2000.0 to Julian Day.
  // Arguments: argJCent : number of Julian centuries since J2000.0
  // Return value: the Julian Day number corresponding to the argJCent value
  return (argJCent*36525.0 + 2451545.0);
}