/*
History
=======
Ver. 1.0  2003-11-08  Initial release by John Land
Ver. 1.1  2003-11-09  Simplified Time String functions
Ver. 1.2  2003-11-10  Added local time computation
Ver. 1.3  2003-11-11  Corrected typos, reversed polarity of time zone numbers
Ver. 1.4  2003-11-13  Prepared code for multilanguage support
Ver. 1.5  2003-11-18  Revised to output individual values, not concatenated strings
Ver. 1.6  2004-04-30  Added 24 hr format output for sun times
Ver. 1.7  2005-10-27  fixed errors in Solar Noon, Equation of Time, and Declination calculations;
                      added computation of Twilight times (civil, nautical, astronomical)
                      and Moon positions and times;
                      simplified code so that changes in display style cause recalculation of
                      only those items that change, rather than always calculate all
                      display styles and then select what to display;
                      moved some date computation routines to DayInfo.js
*/

//=================
// GLOBAL CONSTANTS
//=================

var kSunRefractionOffset   =  0.833;
var kCivilTwilight         =  6.0;
var kNauticalTwilight      = 12.0;
var kAstronomicalTwilight  = 18.0;

//===============================================
// a few general utility functions
//===============================================
function fLeadZero1(argNum) {  // add leading zero to single digit argument
  return ((argNum <0 || argNum > 9 ? '' : '0') + argNum);
}

function fZeroPad(argData, argPlaces) {
  // add specified number of leading zeroes to number
  var vStr       = argData.toString();
  var vPadLength = argPlaces - vStr.length;
  for (var vIndex = 0; vIndex < vPadLength; vIndex++) {
    vStr = '0' + vStr;
  }
  return (vStr);
}

function fSign(argSourceNum, argTargetNum) {
  // apply sign of argSourceNum to argTargetNum
  return (argSourceNum < 0 ? -argTargetNum : argTargetNum);
}

function fIsNumber(argInput) {  // test for {+ -} and decimal
  var vOneDecimal = false;
  var vInputStr = String(argInput);
  for (var vIndex = 0; vIndex < vInputStr.length; vIndex++) {
    var vOneChar = vInputStr.charAt(vIndex);
    if (vIndex == 0 && (vOneChar == '-' || vOneChar == '+')) {
      continue;
    }
    if (vOneChar == '.' && !vOneDecimal) {
      vOneDecimal = true;
      continue;
    }
    if (vOneChar < '0' || vOneChar > '9') {
      return (false);
    }
  }
  return (true);
}

function fCommaSplit(argNumber) {
  // adds commas to any numerical form input
  // based on code provided by Mark Henwood at http://javascript.internet.com/forms/commas.html
  var vTextNumber = String(argNumber);
  if (isNaN(vTextNumber) || vTextNumber == '') {
    return (msgNaN);
  } else {
    var vRegXSplit = new RegExp('([0-9])([0-9][0-9][0-9][,.])');
    var vNumberArray = vTextNumber.split('.');
    vNumberArray[0] += '.';

    do { vNumberArray[0] = vNumberArray[0].replace(vRegXSplit, '$1,$2'); }
    while (vRegXSplit.test(vNumberArray[0]));

    if (vNumberArray.length > 1) {
       return (vNumberArray.join(''));
    } else {
      return (vNumberArray[0].split('.')[0]);
    }
  }
}

function fRadToDeg(argAngleRad) {return ((180.0/Math.PI) * argAngleRad);} // in degrees
function fDegToRad(argAngleDeg) {return ((Math.PI/180.0) * argAngleDeg);} // in radians
function fRev(argAngle)         {return (argAngle-Math.floor(argAngle/360.0)*360.0);}
function fSinD(argAngle)        {return (Math.sin(fDegToRad(argAngle)));}
function fCosD(argAngle)        {return (Math.cos(fDegToRad(argAngle)));}
function fTanD(argAngle)        {return (Math.tan(fDegToRad(argAngle)));}
function fArcSinD(argValue)     {return (fRadToDeg(Math.asin(argValue)));}
function fArcCosD(argValue)     {return (fRadToDeg(Math.acos(argValue)));}
function fArcTan2D(argY, argX)  {return (fRadToDeg(Math.atan(argY/argX))-180.0*(argX<0));}

//=======================================================
// Subroutines used in calculating solar times & position
//=======================================================

function fSunPosition(argJDay, argTimeArray, argZone, argLatitude, argLongitude) {
  //    calculates sun position informtion for the entered date, time &location.
  // Arguments:
  //   argJDay       : current Julian day
  //   argTimeArray  : array of hours, minutes, seconds for a specific time of day
  //   argZone       : -12 to +14
  //   argLatitude   : latitude in decimal format
  //   argLongitude  : longitude in decimal format
  //
  //   Return values:
  //    vEqTime      : Equation of Time in minutes
  //    vRtAscension : Solar Right Ascension in hours, minutes
  //    vSolarDec    : Solar Declination in degrees
  //    Elevation    : Solar elevation in degrees (in vSolarAltAzArray[0])
  //    Azimuth      : Solar azimuth in degrees (in vSolarAltAzArray[1])
  //    vEarthRadVec : Solar distance from Earth, in Astronomical Units

  var vUTCNow = argTimeArray[0] + argTimeArray[1]/60 + (argTimeArray[2]/3600)+ argZone; // in hours
  var vT      = fJCenturyFromJDay(argJDay + vUTCNow/24.0);
  var vEqTime          = fCalcEquationOfTime(vT);
  var vRtAscension     = fCalcSunRtAscension(vT);
  var vSolarDec        = fCalcSunDeclination(vT); // in degrees
  var vSolarAltAzArray = fSolarAltAz(vEqTime,argTimeArray[0],argTimeArray[1],argTimeArray[2],argZone,vSolarDec,argLatitude,argLongitude);
  var vEarthRadVec     = fCalcSunRadVector(vT);
  return ([vEqTime, vRtAscension, vSolarDec, vSolarAltAzArray[0], vSolarAltAzArray[1], vEarthRadVec]);
}

function fCalcGeoMeanLongSun(argJCent) {
  // Calculate the Geometric Mean Longitude of the Sun
  // Arguments: argJCent : number of Julian centuries since J2000.0
  // Return value: the Geometric Mean Longitude of the Sun in degrees
//  var vGeoMeanLong = 280.46646 + argJCent*(36000.76983 + 0.0003032*argJCent);
  var vGeoMeanLong = 280.4664567 + argJCent*(36000.76982779 + 0.0003032028*argJCent); //higher precision
  while(vGeoMeanLong > 360.0) {vGeoMeanLong -= 360.0;}
  while(vGeoMeanLong < 0.0)   {vGeoMeanLong += 360.0;}
  return (vGeoMeanLong); // in degrees
}

function fCalcGeomMeanAnomalySun(argJCent) {
  // Calculate the Geometric Mean Anomaly of the Sun
  // Arguments: argJCent : number of Julian centuries since J2000.0
  // Return value: the Geometric Mean Anomaly of the Sun in degrees
  return (357.52911 + argJCent*(35999.05029 - 0.0001537*argJCent)); // in degrees
}

function fCalcEccentricityEarthOrbit(argJCent) {
  // Calculate the eccentricity of earth's orbit
  // Arguments: argJCent : number of Julian centuries since J2000.0
  // Return value: the unitless eccentricity
//  return (0.016708634 - argJCent*(0.000042037 + 0.0000001267*argJCent));
  return (0.016708634 - argJCent*(0.000042037 + 0.0000001236*argJCent)); //higher precision
}

function fCalcSunEqOfCenter(argJCent) {
  // Calculate the equation of center for the sun
  // Arguments: argJCent : number of Julian centuries since J2000.0
  // Return value:  equation of center for the sun in degrees
  var vM     = fCalcGeomMeanAnomalySun(argJCent);
  var vMrad  = fDegToRad(vM);
  var vSinM  = Math.sin(vMrad);
  var vSin2M = Math.sin(vMrad+vMrad);
  var vSin3M = Math.sin(vMrad+vMrad+vMrad);
  return (vSinM*(1.914602 - argJCent*(0.004817 + 0.000014*argJCent)) +
    vSin2M*(0.019993 - 0.000101*argJCent) + vSin3M*0.000289); // in degrees
}

function fCalcSunTrueLong(argJCent) {
  // Calculate the true longitude of the sun
  // Arguments: argJCent : number of Julian centuries since J2000.0
  // Return value: sun's true longitude in degrees
  return (fCalcGeoMeanLongSun(argJCent) + fCalcSunEqOfCenter(argJCent)); // in degrees
}

function fCalcSunTrueAnomaly(argJCent) {
  // Calculate the true anamoly of the sun
  // Arguments: argJCent : number of Julian centuries since J2000.0
  // Return value: sun's true anamoly in degrees
	return (fCalcGeomMeanAnomalySun(argJCent) + fCalcSunEqOfCenter(argJCent)); // in degrees
}

function fCalcSunRadVector(argJCent) {
// Purpose: calculate the distance to the sun in AU
// Arguments: argJCent : number of Julian centuries since J2000.0
// Return value: sun's radius vector in AUs
	var vV = fCalcSunTrueAnomaly(argJCent);
	var vE = fCalcEccentricityEarthOrbit(argJCent);
	var vR = (1.000001018 * (1 - vE*vE)) / (1 + vE * fCosD(vV));
	return (vR);		// in AU's
}

function fCalcSunApparentLong(argJCent) {
  // Calculate the apparent longitude of the sun
  // Arguments: argJCent : number of Julian centuries since J2000.0
  // Return value: sun's apparent longitude in degrees
  var vO = fCalcSunTrueLong(argJCent);
//  var vOmega = 125.04 - 1934.136*argJCent;
  var vOmega = 125.04452 - 1934.136261*argJCent;
  return (vO - 0.00569 - 0.00478*fSinD(vOmega)); // in degrees; is lambda
}

function fCalcMeanObliquityOfEcliptic(argJCent) {
  // Calculate the mean obliquity of the ecliptic
  // Arguments: argJCent : number of Julian centuries since J2000.0
  // Return value: mean obliquity in degrees
  var vSeconds = 21.448 - argJCent*(46.8150 + argJCent*(0.00059 - argJCent*(0.001813)));
  return (23.0 + (26.0 + (vSeconds/60.0))/60.0); // in degrees; is e0
}

function fCalcObliquityCorrection(argJCent) {
  // Calculate the corrected obliquity of the ecliptic
  // Arguments: argJCent : number of Julian centuries since J2000.0
  // Return value: corrected obliquity in degrees
  var vObliquity = fCalcMeanObliquityOfEcliptic(argJCent);
//  var vOmega = 125.04 - 1934.136*argJCent;
  var vOmega = 125.04452 - 1934.136261*argJCent; //higher precision
  return (vObliquity + 0.00256*fCosD(vOmega)); // in degrees
}

function fCalcSunRtAscension(argJCent) {
  // Calculate the right ascension of the sun
  // Arguments: argJCent : number of Julian centuries since J2000.0
  // Return value: sun's right ascension in scaled hours, minutes
  var vObliquity = fCalcObliquityCorrection(argJCent);
  var vLambda    = fCalcSunApparentLong(argJCent);
  var vTanANum   = fCosD(vObliquity) * fSinD(vLambda);
  var vTanADenom = fCosD(vLambda);
  var vAlpha     = fRadToDeg(Math.atan2(vTanANum, vTanADenom));
  return ((vAlpha/15) + 24.0);  // in scaled hours
}

function fCalcSunDeclination(argJCent) {
  // Calculate the declination of the sun
  // Arguments: argJCent : number of Julian centuries since J2000.0
  // Return value: sun's declination in degrees
  var vObliquity = fCalcObliquityCorrection(argJCent);
  var vLambda    = fCalcSunApparentLong(argJCent);
  var vSinT      = fSinD(vObliquity) * fSinD(vLambda);
  return (fArcSinD(vSinT)); // in degrees; is theta
}

function fCalcEquationOfTime(argJCent) {
  // Calculate the difference between true and mean solar times
  // Arguments: argJCent : number of Julian centuries since J2000.0
  // Return value: equation of time in minutes of time
  var vEpsilon = fCalcObliquityCorrection(argJCent);
  var vL0      = fCalcGeoMeanLongSun(argJCent);
  var vE       = fCalcEccentricityEarthOrbit(argJCent);
  var vM       = fCalcGeomMeanAnomalySun(argJCent);
  var vY       = Math.tan(fDegToRad(vEpsilon)/2.0);
  vY *= vY;

  var vSin2L0  = Math.sin(2.0 * fDegToRad(vL0));
  var vSinM    = fSinD(vM);
  var vCos2L0  = Math.cos(2.0 * fDegToRad(vL0));
  var vSin4L0  = Math.sin(4.0 * fDegToRad(vL0));
  var vSin2M   = Math.sin(2.0 * fDegToRad(vM));

  var vEqTime = vY*vSin2L0 - 2.0*vE*vSinM + 4.0*vE*vY*vSinM*vCos2L0
      - 0.5*vY*vY*vSin4L0 - 1.25*vE*vE*vSin2M;
  return (fRadToDeg(vEqTime)*4.0);  // in minutes of time
}

function fCalcHourAngle(argLatitude, argSolarDec, argOffset) {
  // Calculate the hour angle of the sun for the latitude
  // Arguments:
  //   argLatitude : latitude of observer in degrees
  //   argSolarDec : declination angle of sun in degrees
  //   argOffset   : amount the sun is below the horizon, in degrees; 0.833 for rise/set (accounts for refraction of air),
  //                 6 degrees for civil twilight, 12 degrees for nautical twilight, 18 degrees for astronomical twilight
  // Return value: HA of sunrise in radians; negative gives HA of sunset
  var vLatRad    = fDegToRad(argLatitude);
  var vSolDecRad = fDegToRad(argSolarDec);
  var vHourAngle = (Math.acos(fCosD(90 + argOffset)/
    (Math.cos(vLatRad)*Math.cos(vSolDecRad))-Math.tan(vLatRad)*Math.tan(vSolDecRad)));
  return (vHourAngle); // in radians
}

function fCalcSolNoonUTC(argJCent, argLongitude) {
  // Calculate the Universal Coordinated Time (UTC) of solar
  //    noon for the given day at the given location on earth
  // Arguments:
  //   argJCent     : number of Julian centuries since J2000.0
  //   argLongitude : longitude of observer in degrees
  // Return value: time in minutes from zero Z

	// First pass uses approximate solar noon to calculate eqtime
	var vNoon         = fJCenturyFromJDay(fJDayFromJCentury(argJCent) + argLongitude/360.0);
	var vEqTime       = fCalcEquationOfTime(vNoon);
	var vSolarNoonUTC = 720 + (argLongitude*4) - vEqTime; // minutes

	// Second pass
  var vNewTime      = fJCenturyFromJDay(fJDayFromJCentury(argJCent) - 0.5 + vSolarNoonUTC/1440.0);
  var vEqTime       = fCalcEquationOfTime(vNewTime);
  var vSolNoonUTC   = 720 + (argLongitude*4) - vEqTime; // minutes
  return (vSolNoonUTC);
}

function fSunRiseSetUTC(argJDay, argLatitude, argLongitude, argOffset, argRiseSetFlag) {
  // Calculate the Universal Coordinated Time (UTC) of sunrise
  //      for the given day at the given location on earth
  // Arguments:
  //   argJDay      : Julian day
  //   argLatitude  : latitude of observer in degrees
  //   argLongitude : longitude of observer in degrees
  //   argOffset    : amount the sun is below the horizon, in degrees; 0.833 for rise/set
  //                  (accounts for refraction of air), 6 degrees for civil twilight,
  //                  12 degrees for nautical twilight, 18 degrees for astronomical twilight
  //   argRiseSetFlag : 'set' for sunset, 'rise' for sunrise
  // Return value: time in minutes from zero Z
  var vJCent     = fJCenturyFromJDay(argJDay);

  // Find the time of solar noon at the location, and use that
  // declination. This is better than start of the Julian day
  var vNoonMin   = fCalcSolNoonUTC(vJCent, argLongitude);
  var vNoonTime  = fJCenturyFromJDay(argJDay+vNoonMin/1440.0);

  // First pass to approximate sunrise (using solar noon)
  var vEqTime    = fCalcEquationOfTime(vNoonTime);
  var vSolarDec  = fCalcSunDeclination(vNoonTime);
  switch (argRiseSetFlag) {
    case 'rise' : var vHourAngle =  fCalcHourAngle(argLatitude, vSolarDec, argOffset); break;
    case 'set'  : var vHourAngle = -fCalcHourAngle(argLatitude, vSolarDec, argOffset); break;
  }

  var vDelta     = argLongitude - fRadToDeg(vHourAngle);
  var vTimeDiff  = 4*vDelta;   // in minutes of time
  var vTimeUTC   = 720 + vTimeDiff - vEqTime;   // in minutes

  // Second pass includes fractional jday in gamma calc
  var vNewTime   = fJCenturyFromJDay(fJDayFromJCentury(vJCent) + vTimeUTC/1440.0);
  vEqTime        = fCalcEquationOfTime(vNewTime);
  vSolarDec      = fCalcSunDeclination(vNewTime);
  switch (argRiseSetFlag) {
    case 'rise' : var vHourAngle =  fCalcHourAngle(argLatitude, vSolarDec, argOffset); break;
    case 'set'  : var vHourAngle = -fCalcHourAngle(argLatitude, vSolarDec, argOffset); break;
  }

  vDelta         = argLongitude - fRadToDeg(vHourAngle);
  vTimeDiff      = 4*vDelta;
  return (720 + vTimeDiff - vEqTime); // in minutes
}

function fGetLocation(argDeg, argMin, argSec, argDir) {
  // Returns the decimal value from degrees, minutes and seconds
  // Arguments:
  //   argDeg : degrees
  //   argMin : minutes
  //   argSec : seconds
  //   argDir : direction: S=0, N=1 for latitude, E=0, W=1 for longitude
  var vDecimalLocation = +argDeg + argMin/60 + argSec/3600;
  switch (+argDir) {
    case 0: return (vDecimalLocation);  break; // North or West
    case 1: return (-vDecimalLocation); break; // South or East
  }
}

function fFindPreviousSunrise(argJDay, argLatitude, argLongitude, argOffset) {
  // Calculate the Julian day of the most recent sunrise
  //    starting from the given day at the given location on earth
  // Arguments:
  //   argJDay      : Julian day
  //   argLatitude  : latitude of observer in degrees
  //   argLongitude : longitude of observer in degrees
  //   argOffset    : amount the sun is below the horizon, in degrees; 0.833 for rise/set
  //                  (accounts for refraction of air), 6 degrees for civil twilight,
  //                  12 degrees for nautical twilight, 18 degrees for astronomical twilight
  // Return value: Julian day of the most recent sunrise
  var vJulianDay = argJDay;
  var vTime      = fSunRiseSetUTC(vJulianDay, argLatitude, argLongitude, argOffset, 'rise');
  while(!fIsNumber(vTime)) {
    vJulianDay--;
    vTime = fSunRiseSetUTC(vJulianDay, argLatitude, argLongitude, argOffset, 'rise');
  }
  return (vJulianDay);
}

function fFindPreviousSunset(argJDay, argLatitude, argLongitude, argOffset) {
  // Calculate the Julian day of the most recent sunset
  //    starting from the given day at the given location on earth
  // Arguments:
  //   argJDay      : Julian day
  //   argLatitude  : latitude of observer in degrees
  //   argLongitude : longitude of observer in degrees
  //   argOffset    : amount the sun is below the horizon, in degrees; 0.833 for rise/set
  //                  (accounts for refraction of air), 6 degrees for civil twilight,
  //                  12 degrees for nautical twilight, 18 degrees for astronomical twilight
  // Return value: Julian day of the most recent sunset
  var vJulianDay = argJDay;
  var vTime      = fSunRiseSetUTC(vJulianDay, argLatitude, argLongitude, argOffset, 'set');
  while(!fIsNumber(vTime)) {
    vJulianDay--;
    vTime = fSunRiseSetUTC(vJulianDay, argLatitude, argLongitude, argOffset, 'set');
  }
  return (vJulianDay);
}

function fFindNextSunrise(argJDay, argLatitude, argLongitude, argOffset) {
  // Calculate the Julian day of the next sunrise
  //    starting from the given day at the given location on earth
  // Arguments:
  //   argJDay      : Julian day
  //   argLatitude  : latitude of observer in degrees
  //   argLongitude : longitude of observer in degrees
  //   argOffset    : amount the sun is below the horizon, in degrees; 0.833 for rise/set
  //                  (accounts for refraction of air), 6 degrees for civil twilight,
  //                  12 degrees for nautical twilight, 18 degrees for astronomical twilight
  // Return value: Julian day of the next sunrise
  var vJulianDay = argJDay;
  var vTime      = fSunRiseSetUTC(vJulianDay, argLatitude, argLongitude, argOffset, 'rise');
  while(!fIsNumber(vTime)) {
    vJulianDay++;
    vTime = fSunRiseSetUTC(vJulianDay, argLatitude, argLongitude, argOffset, 'rise');
  }
  return (vJulianDay);
}

function fFindNextSunset(argJDay, argLatitude, argLongitude, argOffset) {
  // Calculate the Julian day of the next sunset
  //    starting from the given day at the given location on earth
  // Arguments:
  //   argJDay      : Julian day
  //   argLatitude  : latitude of observer in degrees
  //   argLongitude : longitude of observer in degrees
  //   argOffset    : amount the sun is below the horizon, in degrees; 0.833 for rise/set
  //                  (accounts for refraction of air), 6 degrees for civil twilight,
  //                  12 degrees for nautical twilight, 18 degrees for astronomical twilight
  // Return value: Julian day of the next sunset
  var vJulianDay = argJDay;
  var vTime      = fSunRiseSetUTC(vJulianDay, argLatitude, argLongitude, argOffset, 'set');
  while(!fIsNumber(vTime)) {
    vJulianDay++;
    vTime = fSunRiseSetUTC(vJulianDay, argLatitude, argLongitude, argOffset, 'set');
  }
  return (vJulianDay);
}

function fSolarAltAz(argEqTime, argHours, argMinutes, argSeconds, argZone, argSolarDec, argLatitude, argLongitude) {
  // compute solar elevation (altitude) & azimuth from time & position information
 	var vTrueSolarTime = fSolarTimeFix(argEqTime, argLongitude, argHours, argMinutes, argSeconds, argZone)

  var vHourAngle = vTrueSolarTime/4.0 - 180.0;
	if (vHourAngle < -180) { vHourAngle += 360.0;	}

	var vHA_Rad = fDegToRad(vHourAngle);
	var vCSZ    = fSinD(argLatitude) * fSinD(argSolarDec) + fCosD(argLatitude) * fCosD(argSolarDec) * Math.cos(vHA_Rad);
  if (vCSZ > 1.0) {
  	vCSZ = 1.0;
  } else if (vCSZ < -1.0) {
  	vCSZ = -1.0;
  }
  var vZenith  = fArcCosD(vCSZ);

  var vAzDenom = (fCosD(argLatitude) * fSinD(vZenith));
  if (Math.abs(vAzDenom) > 0.001) {
		vAzRad     = ((fSinD(argLatitude) * fCosD(vZenith)) - fSinD(argSolarDec)) / vAzDenom;
		if (Math.abs(vAzRad) > 1.0) {
      vAzRad   = (vAzRad < 0) ? -1.0 : 1.0;
		}
  	var vAzimuth = 180.0 - fRadToDeg(Math.acos(vAzRad));
  	if (vHourAngle > 0.0) { vAzimuth = -vAzimuth; }
	} else {
    vAzimuth = (argLatitude > 0.0) ? 180.0 : 0.0;
	}
	if (vAzimuth < 0.0) { vAzimuth += 360.0; }

	var vRefractionCorrection = 0.0;
  var vExoatmElevation      = 90.0 - vZenith;
	if (vExoatmElevation > 85.0) {
		vRefractionCorrection   = 0.0;
	} else {
		vTe = fTanD(vExoatmElevation);
		if (vExoatmElevation > 5.0) {
			vRefractionCorrection = 58.1/vTe - 0.07 / (vTe*vTe*vTe) +	0.000086/(vTe*vTe*vTe*vTe*vTe);
		} else if (vExoatmElevation > -0.575) {
			vRefractionCorrection = 1735.0 + vExoatmElevation *
				(-518.2 + vExoatmElevation * (103.4 + vExoatmElevation * (-12.79 + vExoatmElevation * 0.711)));
		} else {
			vRefractionCorrection = -20.774/vTe;
		}
		vRefractionCorrection   = vRefractionCorrection/3600.0;
	}
	vSolarZenith = vZenith - vRefractionCorrection;

//  	if (vSolarZenith < 108.0) { // astronomical twilight
//   uncomment the next 2 lines, AND comment out the 2 lines after, to display Degrees & Minutes rather than decimal degrees
//   	  vAzimuthDisplay   = fAngleString('DegMin', vAzimuth, true);
//   	  vElevationDisplay = fAngleString('DegMin', 90.0-vSolarZenith, false);
// 	  vElevationDisplay = fTruncateNPlaces(90.0-vSolarZenith, 1) + msgDegrees;
// 	  vAzimuthDisplay   = fTruncateNPlaces(vAzimuth, 1) + msgDegrees;
//   	} else {  // do not report az & el after astronomical twilight
//   	  vAzimuthDisplay   = 'dark';
//   	  vElevationDisplay = 'dark';
//   	}
  return ([90.0-vSolarZenith, vAzimuth]);
}

function fSolarTimeFix(argEqTime, argLongitude, argHours, argMinutes, argSeconds, argZone) {
  // compute time fix from time & position information
  var vSolarTimeFix  = +argEqTime - 4.0*argLongitude + 60.0*argZone;
	var vTrueSolarTime = argHours*60.0 + argMinutes + argSeconds/60 + vSolarTimeFix; // in minutes
	while (vTrueSolarTime > 1440) { vTrueSolarTime -= 1440; }
  return (vTrueSolarTime);
}

//===============================================
// Subroutines used in calculating lunar position
//===============================================

function fMoonPosition(argLatitude, argLongitude, argYear, argMonth, argDay, argHours, argMinutes, argSeconds, argZoneAdjust) {
// fMoonPosition calculates the Moon position, based on Meeus chapter 45
// and the illuminated percentage from Meeus equations 46.4 and 46.1
// Returns an array of numbers:
//   Right Ascension vRtAscension
//   Declination     vDeclination
//   Lunar Distance  vDistance
//   Elevation       vAltAz[0]
//   Azimuth         vAltAz[1]
//   PhaseAngle      vPhaseAngle
//

	// Constants for Earth's Moon, taken from Meeus (1st Edition)
	// Meeus table 45A -- longitude and distance of the Moon
	var vT45AD  = new Array (0, 2, 2, 0, 0, 0, 2, 2, 2, 2,
													 0, 1, 0, 2, 0, 0, 4, 0, 4, 2,
													 2, 1, 1, 2, 2, 4, 2, 0, 2, 2,
													 1, 2, 0, 0, 2, 2, 2, 4, 0, 3,
													 2, 4, 0, 2, 2, 2, 4, 0, 4, 1,
													 2, 0, 1, 3, 4, 2, 0, 1, 2, 2);
	
	var vT45AM  = new Array (0, 0, 0, 0, 1, 0, 0,-1, 0,-1,
													 1, 0, 1, 0, 0, 0, 0, 0, 0, 1,
													 1, 0, 1,-1, 0, 0, 0, 1, 0,-1,
													 0,-2, 1, 2,-2, 0, 0,-1, 0, 0,
													 1,-1, 2, 2, 1,-1, 0, 0,-1, 0,
													 1, 0, 1, 0, 0,-1, 2, 1, 0, 0);
	
	var vT45AMP = new Array (1,-1, 0, 2, 0, 0,-2,-1, 1, 0,
													-1, 0, 1, 0, 1, 1,-1, 3,-2,-1,
													 0,-1, 0, 1, 2, 0,-3,-2,-1,-2,
													 1, 0, 2, 0,-1, 1, 0,-1, 2,-1,
													 1,-2,-1,-1,-2, 0, 1, 4, 0,-2,
													 0, 2, 1,-2,-3, 2, 1,-1, 3,-1);
	
	var vT45AF  = new Array (0, 0, 0, 0, 0, 2, 0, 0, 0, 0,
													 0, 0, 0,-2, 2,-2, 0, 0, 0, 0,
													 0, 0, 0, 0, 0, 0, 0, 0, 2, 0,
													 0, 0, 0, 0, 0,-2, 2, 0, 2, 0,
													 0, 0, 0, 0, 0,-2, 0, 0, 0, 0,
													-2,-2, 0, 0, 0, 0, 0, 0, 0,-2);
	
	var vT45AL  = new Array (6288774, 1274027, 658314, 213618, -185116,
													 -114332,   58793,  57066,  53322,   45758,
														-40923,  -34720, -30383,  15327,  -12528,
														 10980,   10675,  10034,   8548,   -7888,
														 -6766,   -5163,   4987,   4036,    3994,
															3861,    3665,  -2689,  -2602,    2390,
														 -2348,    2236,  -2120,  -2069,    2048,
														 -1773,   -1595,   1215,  -1110,    -892,
															-810,     759,   -713,   -700,     691,
															 596,     549,    537,    520,    -487,
															-399,    -381,    351,   -340,     330,
															 327,    -323,    299,    294,       0);
	
	var vT45AR = new Array(-20905355, -3699111, -2955968, -569925,   48888,
														 -3149,   246158,  -152138, -170733, -204586,
													 -129620,   108743,   104755,   10321,       0,
														 79661,   -34782,   -23210,  -21636,   24208,
														 30824,    -8379,   -16675,  -12831,  -10445,
														-11650,    14403,    -7003,       0,   10056,
															6322,    -9884,     5751,       0,   -4950,
															4130,        0,    -3958,       0,    3258,
															2616,    -1897,    -2117,    2354,       0,
																 0,    -1423,    -1117,   -1571,   -1739,
																 0,    -4421,        0,       0,       0,
																 0,     1165,        0,       0,    8752);
	
	// Meeus table 45B -- latitude of the Moon
	var vT45BD  = new Array (0, 0, 0, 2, 2, 2, 2, 0, 2, 0,
													 2, 2, 2, 2, 2, 2, 2, 0, 4, 0,
													 0, 0, 1, 0, 0, 0, 1, 0, 4, 4,
													 0, 4, 2, 2, 2, 2, 0, 2, 2, 2,
													 2, 4, 2, 2, 0, 2, 1, 1, 0, 2,
													 1, 2, 0, 4, 4, 1, 4, 1, 4, 2);
	
	var vT45BM  = new Array (0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
													-1, 0, 0, 1,-1,-1,-1, 1, 0, 1,
													 0, 1, 0, 1, 1, 1, 0, 0, 0, 0,
													 0, 0, 0, 0,-1, 0, 0, 0, 0, 1,
													 1, 0,-1,-2, 0, 1, 1, 1, 1, 1,
													 0,-1, 1, 0,-1, 0, 0, 0,-1,-2);
	
	var vT45BMP = new Array (0, 1, 1, 0,-1,-1, 0, 2, 1, 2,
													 0,-2, 1, 0,-1, 0,-1,-1,-1, 0,
													 0,-1, 0, 1, 1, 0, 0, 3, 0,-1,
													 1,-2, 0, 2, 1,-2, 3, 2,-3,-1,
													 0, 0, 1, 0, 1, 1, 0, 0,-2,-1,
													 1,-2, 2,-2,-1, 1, 1,-1, 0, 0);
	
	var vT45BF  = new Array (1, 1,-1,-1, 1,-1, 1, 1,-1,-1,
													-1,-1, 1,-1, 1, 1,-1,-1,-1, 1,
													 3, 1, 1, 1,-1,-1,-1, 1,-1, 1,
													-3, 1,-3,-1,-1, 1,-1, 1,-1, 1,
													 1, 1, 1,-1, 3,-1,-1, 1,-1,-1,
													 1,-1, 1,-1,-1,-1,-1,-1,-1, 1);
	
	var vT45BL = new Array (5128122, 280602, 277693, 173237, 55413,
														46271,  32573,  17198,   9266,  8822,
														 8216,   4324,   4200,  -3359,  2463,
														 2211,   2065,  -1870,   1828, -1794,
														-1749,  -1565,  -1491,  -1475, -1410,
														-1344,  -1335,   1107,   1021,   833,
															777,    671,    607,    596,   491,
														 -451,    439,    422,    421,  -366,
														 -351,    331,    315,    302,  -283,
														 -229,    223,    223,   -220,  -220,
														 -185,    181,   -177,    176,   166,
														 -164,    132,   -119,    115,   107);

  // julian date
  var vJDayTime = fJDayFromYMD(argYear, argMonth, argDay) + (argHours + ((argMinutes + argZoneAdjust)/60.0) + (argSeconds/3600.0))/24;
  var vT  = (vJDayTime-2451545.0)/36525;
  var vT2 = vT*vT;
  var vT3 = vT2*vT;
  var vT4 = vT3*vT;
  // Lunar mean longitude L'
  var vLP = 218.3164477 + 481267.88123421*vT - 0.0015786*vT2 + vT3/538841.0  - vT4/65194000.0;
  // Lunar mean elongation
  var vD  = 297.8501921 + 445267.1114034*vT  - 0.0018819*vT2 + vT3/545868.0  - vT4/113065000.0;
  // Solar mean anomaly
  var vM  = 357.5291092 + 35999.0502909*vT   - 0.0001536*vT2 + vT3/24490000.0;
  // Lunar mean anomaly M'
  var vMP = 134.9633964 + 477198.8675055*vT  + 0.0087414*vT2 + vT3/69699.0   - vT4/14712000.0;
  // Lunar argument of latitude
  var vF  = 93.2720950  + 483202.0175233*vT  - 0.0036539*vT2 - vT3/3526000.0 + vT4/863310000.0;

  // Additional arguments
  var vA1 = 119.75 + 131.849*vT;
  var vA2 = 53.09  + 479264.290*vT;
  var vA3 = 313.45 + 481266.484*vT;
  var vE  = 1 - 0.002516*vT - 0.0000074*vT2;
  var vE2 = vE*vE;

  // Sums of periodic terms from Meeus tables 45.A and 45.B
  var vSl = 0;
  var vSr = 0;
  for (var vIndex = 0; vIndex < 60; vIndex++) {
    var vETerm = 1;
    if (Math.abs(vT45AM[vIndex])==1) {vETerm=vE};
    if (Math.abs(vT45AM[vIndex])==2) {vETerm=vE2};
    vSl += vT45AL[vIndex]*vETerm*fSinD(fRev(vT45AD[vIndex]*vD + vT45AM[vIndex]*vM + vT45AMP[vIndex]*vMP + vT45AF[vIndex]*vF));
    vSr += vT45AR[vIndex]*vETerm*fCosD(fRev(vT45AD[vIndex]*vD + vT45AM[vIndex]*vM + vT45AMP[vIndex]*vMP + vT45AF[vIndex]*vF));
  }

  var vSb = 0;
  for (var vIndex = 0; vIndex < 60; vIndex++) {
    var vETerm = 1;
    if (Math.abs(vT45BM[vIndex])==1) vETerm = vE;
    if (Math.abs(vT45BM[vIndex])==2) vETerm = vE2;
      vSb += vT45BL[vIndex]*vETerm*fSinD(fRev(vT45BD[vIndex]*vD + vT45BM[vIndex]*vM + vT45BMP[vIndex]*vMP + vT45BF[vIndex]*vF));
  }

  // Additional additive terms
  vSl = vSl + 3958*fSinD(fRev(vA1))    + 1962*fSinD(fRev(vLP-vF)) + 318*fSinD(fRev(vA2));
  vSb = vSb - 2235*fSinD(fRev(vLP))    +  382*fSinD(fRev(vA3))    + 175*fSinD(fRev(vA1-vF))+
               175*fSinD(fRev(vA1+vF)) +  127*fSinD(fRev(vLP-vMP))- 115*fSinD(fRev(vLP+vMP));

  // geocentric longitude, latitude and distance
  var vGeoMeanLong = fRev(vLP+vSl/1000000.0);
  var vGeoMeanLat  = fRev(vSb/1000000.0);
  vGeoMeanLat      = ((vGeoMeanLat > 180.0) ? vGeoMeanLat - 360 : vGeoMeanLat);
  var vDistance    = Math.round(385000.56 + vSr/1000.0);

  // Obliquity of Ecliptic
  var vObl         = 23.4393 - 3.563E-7*(vJDayTime-2451543.5);

  // right ascension
  var vRtAscension = fRev(fArcTan2D(fSinD(vGeoMeanLong)*fCosD(vObl) - fTanD(vGeoMeanLat)*fSinD(vObl),
                           fCosD(vGeoMeanLong)))/15.0;

  // declination
  var vDeclination = fRev(fArcSinD(fSinD(vGeoMeanLat)*fCosD(vObl) + fCosD(vGeoMeanLat)*fSinD(vObl)*fSinD(vGeoMeanLong)));
  vDeclination     = ((vDeclination > 180.0) ? vDeclination-360: vDeclination);

  // phase angle
  var vPhaseAngle  = fRev(180.0 - vD - 6.289*fSinD(vMP) + 2.1* fSinD(vM) - 1.274*fSinD(2*vD-vMP) -
                     0.658*fSinD(2*vD)-0.214*fSinD(2*vMP) - 0.11*fSinD(vD));

  // convert RA and Declination to elevation and azimuth
  var vLST   = fLocalSidereal(argLongitude, argYear, argMonth, argDay, argHours, argMinutes, argSeconds, argZoneAdjust);
  var vAltAz = fLunarAltAz(vLST, vDeclination, vRtAscension, argLatitude);
  return ([vRtAscension, vDeclination, vDistance, vAltAz[0], vAltAz[1], vPhaseAngle]);
}

function fLunarAltAz(argLSTime, argDeclination, argRtAscension, argLatitude) {
  // convert Right Ascension and Declination to elevation (altitude) and azimuth
  var vX    = fCosD(15.0*(argLSTime-argRtAscension))*fCosD(argDeclination);
  var vY    = fSinD(15.0*(argLSTime-argRtAscension))*fCosD(argDeclination);
  var vZ    = fSinD(argDeclination);
  
  // rotate so vZ is the local zenith
  var vXHor = vX*fSinD(argLatitude) - vZ*fCosD(argLatitude);
  var vYHor = vY;
  var vZHor = vX*fCosD(argLatitude) + vZ*fSinD(argLatitude);
  
  var vElevation = fArcTan2D(vZHor, Math.sqrt(vXHor*vXHor + vYHor*vYHor));
  var vAzimuth   = fRev(fArcTan2D(vYHor,vXHor)+180.0); // so 0 degrees is north
  return ([vElevation, vAzimuth]);
}

function fLocalSidereal(argLongitude, argYear, argMonth, argDay, argHours, argMinutes, argSeconds, argZoneAdjust) {
  // sidereal time in hours for observer
  var vLST = fGreenwichSidereal(argYear, argMonth, argDay);
  vLST    += 1.00273790935*(argHours + ((argMinutes + argZoneAdjust) + (argSeconds/60.0))/60.0);
  vLST    -= argLongitude/15.0;
  while (vLST < 0)  vLST += 24.0;
  while (vLST > 24) vLST -= 24.0;
  return (vLST);
}

function fGreenwichSidereal(argYear, argMonth, argDay) {
  // sidereal time in hours for Greenwich
  var vTime = (fJDayFromYMD(argYear, argMonth, argDay) - 2451545.0)/36525;
  var vGST  = 100.46061837 + vTime*(36000.770053608 + vTime*(0.000387933 - vTime/38710000.0));
  return (fRev(vGST)/15.0);
}

function fMoonRiseSet(argLatitude, argLongitude, argYear, argMonth, argDay, argZoneAdjust) {
  // Moon rise/set search
  // returns an array containing rise and set times or one of the following codes.
  // -1 : means rise or set event not found and Moon was down at 00:00
  // -2 : means rise or set event not found and Moon was up   at 00:00

  var vRiseSetArray = new Array();

  // vELH is the elevation at the hour vELHdone is true, if vELH calculated
  var vELH     = new Array();
  var vELHdone = new Array();
  for (var vIndex = 0; vIndex <= 24; vIndex++) {
    vELHdone[vIndex] = false;
  }

  vMoonArray  = fMoonPosition(argLatitude, argLongitude, argYear, argMonth, argDay, 0, 0, 0, argZoneAdjust);

  vELH[0]     = vMoonArray[3]; // vMoonArray[3] is the elevation of the Moon
  vELHdone[0] = true;

  if (vELH[0] >= 0.0) {  // set the return code to allow for always up or never rises
    vRiseSetArray = [-2,-2];
  } else {
    vRiseSetArray = [-1,-1];
  }

  vMoonArray   = fMoonPosition(argLatitude, argLongitude, argYear, argMonth, argDay, 24, 0, 0, argZoneAdjust);
  vELH[24]     = vMoonArray[3];
  vELHdone[24] = true;

  // search for moonrise and set
  for (var vRise = 0; vRise <= 1; vRise++) {
    var vFound  = false;
    var vHFirst =  0;
    var vHLast  = 24;

    // Try a binary search on the hours for speed
    while (Math.ceil((vHLast - vHFirst)/2) > 1) {
      var vHMid = vHFirst + Math.round((vHLast - vHFirst)/2);
      if (!vELHdone[vHMid]) {
        vMoonArray      = fMoonPosition(argLatitude, argLongitude, argYear, argMonth, argDay, vHMid, 0, 0, argZoneAdjust);
        vELH[vHMid]     = vMoonArray[3];
        vELHdone[vHMid] = true;
      }
      if (((vRise == 0) && (vELH[vHFirst] <= 0.0) && (vELH[vHMid] >= 0.0)) ||
         ((vRise  == 1) && (vELH[vHFirst] >= 0.0) && (vELH[vHMid] <= 0.0))) {
         vHLast  = vHMid;
         vFound  = true;
         continue;
      }
      if (((vRise == 0) && (vELH[vHMid] <= 0.0) && (vELH[vHLast] >= 0.0)) ||
         ((vRise  == 1) && (vELH[vHMid] >= 0.0) && (vELH[vHLast] <= 0.0))) {
         vHFirst = vHMid;
         vFound  = true;
         continue;
      }
      break;
    }

    // If the binary search did not find a 1 hour interval, try this
    if ((vHLast - vHFirst) > 1) {
      for (var vIndex = vHFirst; vIndex < vHLast; vIndex++) {
        vFound = false;
        if (!vELHdone[vIndex+1]) {
          vMoonArray = fMoonPosition(argLatitude, argLongitude, argYear, argMonth, argDay, vIndex+1, 0, 0, argZoneAdjust);
          vELH[vIndex+1]     = vMoonArray[3];
          vELHdone[vIndex+1] = true;
        }
        if (((vRise == 0) && (vELH[vIndex] <= 0.0) && (vELH[vIndex+1] >= 0.0)) ||
            ((vRise == 1) && (vELH[vIndex] >= 0.0) && (vELH[vIndex+1] <= 0.0))) {
          vHFirst = vIndex;
          vHLast  = vIndex+1;
          vFound  = true;
          break;
        }
      }
    }

    // simple linear interpolation for the minutes
    if (vFound) {
      var vELFirst = vELH[vHFirst];
      var vELLast  = vELH[vHLast];
      vMoonArray = fMoonPosition(argLatitude, argLongitude, argYear, argMonth, argDay, vHFirst, 30, 0, argZoneAdjust);
      if ((vRise == 0) && (vMoonArray[3] <= 0.0)) {vHFirst = vHFirst+0.5; vELFirst = vMoonArray[3];}
      if ((vRise == 0) && (vMoonArray[3]  > 0.0)) {vHLast  = vHFirst+0.5; vELLast  = vMoonArray[3];}
      if ((vRise == 1) && (vMoonArray[3] <= 0.0)) {vHLast  = vHFirst+0.5; vELLast  = vMoonArray[3];}
      if ((vRise == 1) && (vMoonArray[3]  > 0.0)) {vHFirst = vHFirst+0.5; vELFirst = vMoonArray[3];}
      var vELD = Math.abs(vELFirst) + Math.abs(vELLast);
      vRiseSetArray[vRise] = vHFirst + (vHLast - vHFirst)*Math.abs(vELFirst)/vELD;
    }
  }
  return (vRiseSetArray); // array of hours, decimal minutes
}

//=================================
// time string formatting functions
//=================================
function fMinutesToHM(argTimeMin) {
  // Convert time of day in integer minutes to Hours, Minutes
  // Arguments: argTimeMin : time in integer minutes
  // Return value: Hours, Minutes,
  var vHours   = Math.floor(argTimeMin/60);
  var vMinutes = (60 + (argTimeMin)%60)%60;
  return [vHours, vMinutes];
}

function fMinutesToHMS(argTimeMin) {
  // Convert time of day in decimal minutes to Hours, Minutes, Seconds
  // Arguments: argTimeMin : time of day in decimal minutes
  // Return value: Hours, Minutes, Seconds
  var vFloatHour   = argTimeMin / 60;
  var vHours       = Math.floor(vFloatHour);
  var vFloatMinute = 60*(vFloatHour - vHours);
  var vMinutes     = Math.floor(vFloatMinute);
  var vFloatSec    = 60*(vFloatMinute - vMinutes);
  var vSeconds     = Math.floor(vFloatSec);
  return [vHours, vMinutes, vSeconds];
}

function fHMFormat(argHours, argMins) {
  // select a time format based on a global variable
  switch (+vglobalClockStyle) {
    case 0:   // fix hours display to be 1-12, & midnight display to be 12 hours, not 0 hours
      var vDisplayHours = (argHours > 12) ? argHours-12 : (argHours == 0) ? 12 : argHours;
      return (fHMTimeString(vDisplayHours, argMins, false) + ((argHours >= 12) ? ' '+msg_pm:' '+msg_am));
      break;

    default:  // 24 hr mode
      return (fHMTimeString(argHours, argMins, true));
      break;
  }
}

function fHMTimeString(argHours, argMins, argPadHourFlag) {
  // convert numeric hours, minutes, to display string,
  // with option to zero pad leading digit (argPadHourFlag=true)
  return ((argPadHourFlag ? fLeadZero1(argHours) : argHours) +':'+ fLeadZero1(argMins));
}

function fHMSTimeString(argHours, argMins, argSecs, argPadHourFlag) {
  // convert numeric hours, minutes, seconds to display string,
  // with option to zero pad leading digit (argPadHourFlag=true)
  return ((argPadHourFlag ? fLeadZero1(argHours) : argHours) +':'+ fLeadZero1(argMins) +':'+ fLeadZero1(argSecs));
}

function fHMS_String(argAngle) {
  // convert angle hours to a signed, colon seperated string
  var vHours   = Math.abs(argAngle);
  var vMinutes = Math.round(60.0*(vHours-Math.floor(vHours)));
  vHours       = Math.floor(vHours);
  if (vMinutes >= 60) {
    vHours++;
    vMinutes -= 60;
  }
  if (vHours >= 24) {
    vHours -= 24;
  }
  var vAngleStr = ((argAngle < 0) ? '-' : '') + fHMTimeString(vHours, vMinutes, true);
  return (vAngleStr);
}

function fTimeStringPlain(argTimeMin, argDisplayStyle) {
  // Convert time of day in minutes to a zero-padded string
  //    suitable for printing to the form text fields
  // Arguments:
  //   argTimeMin      : time of day in decimal minutes
  //   argDisplayStyle : string defining a selected style:
  //     "24Hr"     :  HH:MM
  //     "AMPM"     :  H:MM am/pm
  // Return value: string of the format HH:MM:SS, minutes and seconds are zero padded
  var vTimeInfo = fMinutesToHMS(argTimeMin);
  var vHour     = vTimeInfo[0];
  var vMinute   = vTimeInfo[1];
  var vSecond   = vTimeInfo[2];
  
  switch (argDisplayStyle) {
    case 'AMPM' :
      var vDisplayHours = (vHour > 12) ? vHour - 12 : (vHour == 0) ? 12 : vHour;
      var vTimeStr = fHMSTimeString(vDisplayHours, vMinute, vSecond, false) + ' '
                     + ((vHour >= 12) ? msg_pm : msg_am);
      break;
    case '24Hr' :
      var vTimeStr = fHMSTimeString(vHour, vMinute, vSecond, true);
      break;
  }
  return (vTimeStr);
}

function fTimeStrSelector(argTimeMin, argJDay, argDisplayStyle) {
  // Convert time of day in decimal minutes to a zero-padded string
  //    suitable for printing to the form text fields.
  // Arguments:
  //   argTimeMin      : time of day in minutes
  //   argJDay         : Julian day
  //   argDisplayStyle : string defining a selected style:
  //     "24Hr"     :  H:MM plus date DDMon if time crosses a day boundary
  //     "24HrDate" :  H:MM plus date DDMon
  //     "AMPM"     :  H:MM am/pm plus date DDMon if time crosses a day boundary
  //     "AMPMDate" :  H:MM am/pm plus date DDMon
  // Return value: string of the format designated by argDisplayStyle
  var vTimeInfo = fMinutesToHMS(argTimeMin);
  var vHour     = vTimeInfo[0];
  var vMinute   = vTimeInfo[1];
  var vSecond   = vTimeInfo[2];

  var vJulianDay = argJDay;
  var vDayChange = false;

  vMinute += (vSecond >= 30)? 1 : 0;

  if (vMinute >= 60) {
    vMinute -= 60;
    vHour++;
  }

  if (vHour > 23) {
    vHour -= 24;
    vJulianDay++;
    vDayChange = true;
  }
  if (vHour < 0) {
    vHour += 24;
    vJulianDay--;
    vDayChange = true;
  }
  switch (argDisplayStyle) {
    case '24HrDate' :
      return (fHMTimeString(vHour, vMinute, true) + ' ' + fDateMonthStrFromJDay(vJulianDay));
      break;

    case '24Hr' :
      var vTimeStr = fHMTimeString(vHour, vMinute, true);
      if (vDayChange) {
        vTimeStr += ' ' + fDateMonthStrFromJDay(vJulianDay);
      }
      return (vTimeStr);
      break;

    case 'AMPM' :
    case 'AMPMDate' :
      var vDisplayHours = (vHour > 12) ? vHour - 12 : (vHour == 0) ? 12 : vHour;
      var vTimeStr      = fHMTimeString(vDisplayHours, vMinute, false) + ' '
                     + ((vHour >= 12) ? msg_pm : msg_am);
      switch (argDisplayStyle) {
        case 'AMPM' :
          if (vDayChange) {
            vTimeStr += ' ' + fDateMonthStrFromJDay(vJulianDay);
          }
          return (vTimeStr);
          break;
        case 'AMPMDate' :
          return (vTimeStr + ' ' + fDateMonthStrFromJDay(vJulianDay));
          break;
      }
      break;
  }
}

function fFormatTime(argArray, argZoneAdjust, argJDay, argStyle) {
  var vTime = 0;
  switch (+argStyle) {
    case 0:  // am/pm time mode; fix hours display to be 1-12, & midnight display to be 12 hours, not 0 hours
      var vDisplayHours = (argArray[0] > 12) ? argArray[0] - 12 : (argArray[0] == 0) ? 12 : argArray[0];
      vTime = fHMSTimeString(vDisplayHours, argArray[1], argArray[2], false) + ((argArray[0] >= 12) ? ' '+msg_pm:' '+msg_am);
      break;
    case 1:
      vTime = fHMSTimeString(argArray[0], argArray[1], argArray[2], true);
      break;
    case 2:
      vTime = 60*argArray[0] + argArray[1] + argArray[2]/60 + argZoneAdjust; // in minutes; factor out DST
      vTime = msg_utc+ ' ' + fTimeStrSelector(vTime, argJDay, '24Hr')
      break;
  }
  return (vTime);
}

function fGetTimeArray(argZoneAdjust) {
  // return hours, minutes, seconds
  var vNow = new Date();
  with (vNow) {
    // get UTC time; add in the selected offset/adjustment to find the time in the selected time zone
    var vUTCTime   = (60*getUTCHours()) + getUTCMinutes() + (getUTCSeconds()/60) - argZoneAdjust; // in minutes;
    var vTimeArray = fMinutesToHMS(vUTCTime);

    if (vTimeArray[0] > 23) {
      vTimeArray[0] -= 24;
    }
    if (vTimeArray[0] < 0) {
      vTimeArray[0] += 24;
    }
    vTimeArray[0] %= 24 //adjust '24' to 'O'

  }
  return ([vTimeArray[0], vTimeArray[1], vTimeArray[2]]);
}

function fTruncateNPlaces(argNum, argPlaces) {
  // round a number at a specified number of decimal places
  var vFactor = 1;
  for (var vIndex = 1; vIndex <= argPlaces; vIndex++) {
    vFactor *= 10;
  }
  return ((Math.round(vFactor * argNum))/vFactor);
}

function fAngleString(argType, argAngle, argCircleState) {
  // return angle as degrees:minutes
  // set argCircleState to true for a range between 0 and 360, and false for -90 to +90
  var vAngleStr = '';
  switch (argType) {
    case 'Decimal' :
      vAngleStr = fTruncateNPlaces(argAngle, 2) + msgDegrees;
      break;

    case 'DegMin'  :
      var vAngle = Math.round(argAngle*60)/60;
      var vDeg   = Math.abs(vAngle);
      var vMin   = Math.round(60.0*(vDeg-Math.floor(vDeg)));
      if (vMin >= 60) {
         vDeg++;
         vMin = 0;
      }

      if (!argCircleState) vAngleStr += (vAngle < 0 ? '-' : '+');
      if (argCircleState)  vAngleStr += ((Math.floor(vDeg) < 100) ? '0' : '' );
      vAngleStr += fLeadZero1(Math.floor(vDeg)) + msgDegrees +' ' + fLeadZero1(vMin) + '\'';  // note the escape slash
      break;
  }
  return (vAngleStr);
}

//==============================================================================
//==============================================================================
function fMainSunCalc(argYear, argMonth, argDay, argZone, argZnAdj, argDST,
  argLatDeg,  argLatMin,  argLatSec,  argLatDir,
  argLongDeg, argLongMin, argLongSec, argLongDir,
  argHours,   argMinutes, argSeconds, argTimeOverride,
  argFrmLocation, argOffset, argTimeStyle) {
//==============================================================================
//==============================================================================
  // Type:    Main Sun function called by form controls
  //    calculates time of sunrise and sunset for the entered date
  //    and location.  In the special cases near earth's poles,
  //    the date of nearest sunrise and sunset are reported.
  // Arguments:
  //   argYear    : 4-digit year
  //   argMonth   : 1-12   must be converted from 0-11 range before calling this function
  //   argDay     : 1-31
  //   argZone    : -12 to +14
  //   argZnAdj   : 0, +.25, +.50, or +.75
  //   argDST     : 0 or 60 minutes
  //   argLatDeg  : 0-90
  //   argLatMin  : 0-59
  //   argLatSec  : 0-59
  //   argLatDir  : E=0 or W=1
  //   argLongDeg : 0-180
  //   argLongMin : 0-59
  //   argLongSec : 0-59
  //   argLongDir : N=0 or S=1
  //   argHours   : 0-24 override Hours for position information
  //   argMinutes : 0-59 override Minutess for position information
  //   argSeconds : 0-59 override Seconds for position information
  //   argTimeOverride : 0-1 checkbox value to determine if override time should be used
  //   argFrmLocation  : form for receiving corrected inputs
  //   argOffset       : amount the sun is below the horizon, in degrees; 0.833 for rise/set
  //                     (accounts for refraction of air), 6 degrees for civil twilight,
  //                     12 degrees for nautical twilight, 18 degrees for astronomical twilight
  //   argTimeStyle    : style for date display - 0=am/pm  1=24hr  2=UTC
  //   Return values:
  //    vEqTimeDisplay        : Equation of Time in minutes
  //    vDeclinationDisplay   : Solar Declination in degrees
  //    vRtAscensionDisplay   : Solar Right Ascension in hours, minutes
  //    vAzimuthDisplay       : Solar azimuth in degrees
  //    vElevationDisplay     : Solar elevation in degrees
  //    vDistanceDisplay      : Solar distance from Earth, in Astronomical Units
  //    vSunriseStrDisplay    : Local Sunrise or selected Twilight time string in selected format
  //    vSolNoonStrDisplay    : Local Solar Noon time string in selected format
  //    vSunsetStrDisplay     : Local Sunset or selected Twilight time string in selected format
  //    vDayLengthDisplay     : Length of day in hours, minutes
  //    vTimeDisplay          : Time at which Solar position calculations were made
  //
  // return variables:
  var vEqTimeDisplay      ='';
  var vDeclinationDisplay ='';
  var vRtAscensionDisplay ='';
  var vAzimuthDisplay     ='';
  var vElevationDisplay   ='';
  var vDistanceDisplay    ='';
  var vSunriseStrDisplay  ='';
  var vSolNoonStrDisplay  ='';
  var vSunsetStrDisplay   ='';
  var vDayLengthDisplay   ='';
  var vTimeDisplay        ='';

  if (argYear <1000 || argYear >3000) {
    //alert(msgOutOfSkyTimesRange);
    return ([vEqTimeDisplay, vRtAscensionDisplay, vDeclinationDisplay, vElevationDisplay, vAzimuthDisplay,
  vDistanceDisplay, vSunriseStrDisplay, vSolNoonStrDisplay, vSunsetStrDisplay, vDayLengthDisplay, vTimeDisplay]);
  }

  var vLatitude  = fGetLocation(argLatDeg, argLatMin, argLatSec, argLatDir);
  var vLongitude = fGetLocation(argLongDeg, argLongMin, argLongSec, argLongDir);

  if (vLatitude < -89.8 || vLatitude > 89.8) {
    alert(msgLatTooCloseToPoles);
    vLatitude = (argLatDir == 0 ? 89.8 : -89.8);
    with (argFrmLocation) {  //  Note call back to form
      frmLatDegInput.value = 89;
      frmLatMinInput.value = 48;
      frmLatSecInput.value = 0;
    }
  }

  // Calculate the Julian date and century for sun data computations
  var vJDay  = fJDayFromYMD(argYear, argMonth, argDay);
  var vJCent = fJCenturyFromJDay(vJDay);

  // Calculate sunrise for this date; if not found, set flag vNoSunriseFlag
  var vNoSunriseFlag  = false;
  var vSunriseTimeUTC = fSunRiseSetUTC(vJDay, vLatitude, vLongitude, argOffset, 'rise');
  if (!fIsNumber(vSunriseTimeUTC)) {
    vNoSunriseFlag = true;
  }

  // Calculate sunset for this date; if not found, set flag vNoSunsetFlag
  var vNoSunsetFlag  = false;
  var vSunsetTimeUTC = fSunRiseSetUTC(vJDay, vLatitude, vLongitude, argOffset, 'set');
  if (!fIsNumber(vSunsetTimeUTC)) {
    vNoSunsetFlag = true;
  }

  var vZone = Math.abs(argZone) + (+argZnAdj);
  vZone     = (argZone < 0 ? vZone : -vZone);         // reverse sign
  var vZoneAdjust = 60*vZone - argDST;  // in minutes

  if (!vNoSunriseFlag) { // Sunrise was found
    var vSunriseTimeLST = vSunriseTimeUTC - vZoneAdjust; // in minutes
    switch (argTimeStyle) {
      case 0: vSunriseStrDisplay = fTimeStrSelector(vSunriseTimeLST,               vJDay, 'AMPM'); break;
      case 1: vSunriseStrDisplay = fTimeStrSelector(vSunriseTimeLST,               vJDay, '24Hr'); break;
      case 2: vSunriseStrDisplay = msg_utc+' ' + fTimeStrSelector(vSunriseTimeUTC, vJDay, '24Hr'); break;
    }
    if (argOffset == kSunRefractionOffset) { // compute azimuth for sunrise; don't do for twilights
      var vNowArray      = fMinutesToHMS (vSunriseTimeLST);
      var vSunArray      = fSunPosition(vJDay, vNowArray, vZone, vLatitude, vLongitude);
      vSunriseStrDisplay = vSunriseStrDisplay + ' (' + Math.round(vSunArray[4]) + msgDegrees + ')';
    }
  }

  if (!vNoSunsetFlag)  { // Sunset was found
    var vSunsetTimeLST  = vSunsetTimeUTC - vZoneAdjust; // in minutes
    switch (argTimeStyle) {
      case 0: vSunsetStrDisplay = fTimeStrSelector(vSunsetTimeLST,               vJDay, 'AMPM'); break;
      case 1: vSunsetStrDisplay = fTimeStrSelector(vSunsetTimeLST,               vJDay, '24Hr'); break;
      case 2: vSunsetStrDisplay = msg_utc+' ' + fTimeStrSelector(vSunsetTimeUTC, vJDay, '24Hr'); break;
    }
    if (argOffset == kSunRefractionOffset) { // compute azimuth for sunset; don't do for twilights
      var vNowArray      = fMinutesToHMS (vSunsetTimeLST);
      var vSunArray      = fSunPosition(vJDay, vNowArray, vZone, vLatitude, vLongitude);
      vSunsetStrDisplay  = vSunsetStrDisplay + ' (' + Math.round(vSunArray[4]) + msgDegrees + ')';
    }
  }

  if (!vNoSunriseFlag && !vNoSunsetFlag) { // Sunrise and Sunset were both found
    vDayLengthDisplay = fMinutesToHMS(vSunsetTimeLST - vSunriseTimeLST);
    vDayLengthDisplay = vDayLengthDisplay[0] +  ' ' + msg_H + ' ' + fLeadZero1(vDayLengthDisplay[1]) +  ' ' + msg_M;
  }

  // Calculate solar noon for this date
  var vSolNoonUTC = fCalcSolNoonUTC(vJCent, vLongitude);
  var vSolNoonLST = vSolNoonUTC - vZoneAdjust; // in minutes
  switch (argTimeStyle) {
    case 0: vSolNoonStrDisplay = fTimeStringPlain(vSolNoonLST, 'AMPM'); break;
    case 1: vSolNoonStrDisplay = fTimeStringPlain(vSolNoonLST, '24Hr'); break;
    case 2: vSolNoonStrDisplay = msg_utc+' ' + fTimeStringPlain(vSolNoonUTC, '24Hr'); break;
  }

  var vDOYNum = fDayOfYear(argYear, argMonth, argDay);

  if(vNoSunriseFlag) { // report special cases of no sunrise
//     if (argOffset != kSunRefractionOffset) { // skip corrections for twilight values
//       vSunriseStrDisplay = msgNotApplicable;
//     } else {
      // if Northern hemisphere and spring or summer, OR
      // if Southern hemisphere and fall or winter,
      // use previous sunrise and next sunset
      if (((vLatitude > 66.4) && (vDOYNum > 79)  && (vDOYNum < 267)) ||
         ((vLatitude < -66.4) && ((vDOYNum < 83) || (vDOYNum > 263)))) {
        vNewJDay   = fFindPreviousSunrise(vJDay, vLatitude, vLongitude, argOffset);
        vSunriseTimeUTC = fSunRiseSetUTC(vNewJDay, vLatitude, vLongitude, argOffset, 'rise');
        if (+argTimeStyle == 2) {vSunriseStrDisplay = '< ' + msg_utc+' ' + fTimeStrSelector(vSunriseTimeUTC, vNewJDay, '24HrDate');}

        vNewLSTime = vSunriseTimeUTC - vZoneAdjust;
        if (vNewLSTime > 1440) { // 1440 is the number of minutes in a day
          vNewLSTime -= 1440;
          vNewJDay++;
        }
        if (vNewLSTime < 0) {
          vNewLSTime += 1440;
          vNewJDay--;
        }
        switch (argTimeStyle) {
          case 0: vSunriseStrDisplay = '< ' + fTimeStrSelector(vNewLSTime, vNewJDay, 'AMPMDate'); break;
          case 1: vSunriseStrDisplay = '< ' + fTimeStrSelector(vNewLSTime, vNewJDay, '24HrDate'); break;
        }
        vDayLengthDisplay = '24 '+ msg_H;
      }

      // if Northern hemisphere and fall or winter, OR
      // if Southern hemisphere and spring or summer,
      // use next sunrise and previous sunset
      else if (((vLatitude > 66.4)  && ((vDOYNum < 83) || (vDOYNum > 263))) ||
               ((vLatitude < -66.4) && (vDOYNum > 79)  && (vDOYNum < 267))) {
        vNewJDay = fFindNextSunrise(vJDay, vLatitude, vLongitude, argOffset);
        vSunriseTimeUTC = fSunRiseSetUTC(vNewJDay, vLatitude, vLongitude, argOffset, 'rise');
        if (+argTimeStyle == 2) {vSunriseStrDisplay = '> ' + msg_utc+' ' + fTimeStrSelector(vSunriseTimeUTC, vNewJDay, '24HrDate');}

        vNewLSTime = vSunriseTimeUTC - vZoneAdjust;
        if (vNewLSTime > 1440) {
          vNewLSTime -= 1440;
          vNewJDay++;
        }
        if (vNewLSTime < 0) {
          vNewLSTime += 1440;
          vNewJDay--;
        }
        switch (argTimeStyle) {
          case 0: vSunriseStrDisplay = '> ' + fTimeStrSelector(vNewLSTime, vNewJDay, 'AMPMDate'); break;
          case 1: vSunriseStrDisplay = '> ' + fTimeStrSelector(vNewLSTime, vNewJDay, '24HrDate'); break;
        }

        vSolNoonStrDisplay = msgNotApplicable;
        vDayLengthDisplay  = '0 '+ msg_H;
      } else {
        vSunriseStrDisplay = '';
//      alert(msgNoSunriseFound);
      }
//     }
  }

  if(vNoSunsetFlag) { // report special cases of no sunset
//     if (argOffset != kSunRefractionOffset) { // skip corrections for twilight values
//       vSunsetStrDisplay = msgNotApplicable;
//     } else {
      // if Northern hemisphere and spring or summer, OR
      // if Southern hemisphere and fall or winter,
      // use previous sunrise and next sunset
      if (((vLatitude > 66.4)  &&  (vDOYNum > 79) && (vDOYNum < 267)) ||
          ((vLatitude < -66.4) && ((vDOYNum < 83) || (vDOYNum > 263)))) {
        vNewJDay = fFindNextSunset(vJDay, vLatitude, vLongitude, argOffset);
        vSunsetTimeUTC = fSunRiseSetUTC(vNewJDay, vLatitude, vLongitude, argOffset, 'set');
        if (argTimeStyle==2) {vSunsetStrDisplay = '> ' + msg_utc+' ' + fTimeStrSelector(vSunsetTimeUTC, vNewJDay, '24HrDate');}

        vNewLSTime = vSunsetTimeUTC - vZoneAdjust;
        if (vNewLSTime > 1440) {
          vNewLSTime -= 1440;
          vNewJDay++;
        }
        if (vNewLSTime < 0) {
          vNewLSTime += 1440;
          vNewJDay--;
        }
        switch (argTimeStyle) {
          case 0: vSunsetStrDisplay = '> ' + fTimeStrSelector(vNewLSTime, vNewJDay, 'AMPMDate'); break;
          case 1: vSunsetStrDisplay = '> ' + fTimeStrSelector(vNewLSTime, vNewJDay, '24HrDate'); break;
        }
        vDayLengthDisplay = '24 '+ msg_H;
        
      }

      // if Northern hemisphere and fall or winter, OR
      // if Southern hemisphere and spring or summer,
      // use next sunrise and last sunset
      else if (((vLatitude > 66.4)  && ((vDOYNum < 83) || (vDOYNum > 263))) ||
               ((vLatitude < -66.4) &&  (vDOYNum > 79) && (vDOYNum < 267))) {
        vNewJDay = fFindPreviousSunset(vJDay, vLatitude, vLongitude, argOffset);
        vSunsetTimeUTC = fSunRiseSetUTC(vNewJDay, vLatitude, vLongitude, argOffset, 'set');
        if (+argTimeStyle == 2) {vUTCSunsetStrDisplay = '< ' + msg_utc+' ' + fTimeStrSelector(vSunsetTimeUTC, vNewJDay, '24HrDate');}

        vNewLSTime = vSunsetTimeUTC - vZoneAdjust;
        if (vNewLSTime > 1440) {
          vNewLSTime -= 1440;
          vNewJDay++;
        }
        if (vNewLSTime < 0) {
          vNewLSTime += 1440;
          vNewJDay--;
        }
        switch (argTimeStyle) {
          case 0: vSunsetStrDisplay = '< ' + fTimeStrSelector(vNewLSTime, vNewJDay, 'AMPMDate'); break;
          case 1: vSunsetStrDisplay = '< ' + fTimeStrSelector(vNewLSTime, vNewJDay, '24HrDate'); break;
        }

        vSolNoonStrDisplay = msgNotApplicable;
        vDayLengthDisplay  = '0 '+ msg_H;
      } else {
        vSunsetStrDisplay = '';
//      alert(msgNoSunsetFound);
      }
//     }
  }

//===============================================================
// do this part only when computing sunrise/sunset, not twilights
//===============================================================
  if (+argOffset == kSunRefractionOffset) {

    if (argTimeOverride) { // if set, use the time values that the user provided
      var vNowArray = fMinutesToHM(argHours*60.0 + (+argMinutes) - argDST);
      vNowArray.push(+argSeconds);  // add seconds as array element
    } else {
      var vNowArray = fGetTimeArray(vZoneAdjust + (+argDST));
    }
    var vDisplayArray    = fMinutesToHMS (vNowArray[0]*60.0 + vNowArray[1] + vNowArray[2]/60.0 + (+argDST));
    var vTimeDisplay     = fFormatTime(vDisplayArray, vZoneAdjust, vJDay, argTimeStyle);

    var vSunArray        = fSunPosition(vJDay, vNowArray, vZone, vLatitude, vLongitude);

    vEqTimeDisplay       = fTruncateNPlaces(vSunArray[0], 2) + ' ' + msg_Min;
    vRtAscensionDisplay  = fHMS_String(vSunArray[1]);
    vDeclinationDisplay  = fAngleString('DegMin', vSunArray[2], false);
	  vElevationDisplay    = fTruncateNPlaces(vSunArray[3], 1) + msgDegrees;
	  vAzimuthDisplay      = fTruncateNPlaces(vSunArray[4], 1) + msgDegrees;
    vDistanceDisplay     = fTruncateNPlaces(vSunArray[5], 4) + ' AU';
//   vDeclinationDisplay = fTruncateNPlaces(vSolarDec, 2) + msgDegrees;  // alternative format
  }

  return ([vEqTimeDisplay, vRtAscensionDisplay, vDeclinationDisplay, vElevationDisplay, vAzimuthDisplay,
  vDistanceDisplay, vSunriseStrDisplay, vSolNoonStrDisplay, vSunsetStrDisplay, vDayLengthDisplay, vTimeDisplay]);
}

//==============================================================================
//==============================================================================
function fMainMoonCalc(argYear, argMonth, argDay, argZone, argZnAdj, argDST,
  argLatDeg,  argLatMin,  argLatSec,  argLatDir,
  argLongDeg, argLongMin, argLongSec, argLongDir,
  argHours,   argMinutes, argSeconds, argTimeOverride,
  argFrmLocation, argTimeStyle) {
//==============================================================================
//==============================================================================
  // Type:    Main Moon function called by form controls
  // calculate time of Moonrise and Moonset for the entered date and location.
  // Arguments:
  //   argYear    : 4-digit year
  //   argMonth   : 1-12   must be converted from 0-11 range before calling this function
  //   argDay     : 1-31
  //   argZone    : -12 to +14
  //   argZnAdj   : 0, +.25, +.50, or +.75
  //   argDST     : 0 or 60 minutes
  //   argLatDeg  : 0-90
  //   argLatMin  : 0-59
  //   argLatSec  : 0-59
  //   argLatDir  : E=0 or W=1
  //   argLongDeg : 0-180
  //   argLongMin : 0-59
  //   argLongSec : 0-59
  //   argLongDir : N=0 or S=1
  //   argHours   : 0-24 override Hours for position information
  //   argMinutes : 0-59 override Minutess for position information
  //   argSeconds : 0-59 override Seconds for position information
  //   argTimeOverride : 0-1 checkbox value to determine if override time should be used
  //   argFrmLocation  : form for receiving corrected inputs
  //   argTimeStyle    : style for date display - 0=am/pm  1=24hr  2=UTC
  //   Return values:
  //    vRightAscensionDisplay :
  //    vDeclinationDisplay    :
  //    vElevationDisplay       :
  //    vAzimuthDisplay        :
  //    vDistanceDisplay       :
  //    vPhaseAngleDisplay     : Phase Angle
  //    vIlluminationDisplay   : Percent of illumination of the Moon
  //    vMoonRiseDisplay       : Local Moonrise time string in selected format
  //    vMoonSetDisplay        : Local Moonset time string in selected format
  //    vTimeDisplay           : Time at which Lunar position calculations were made
  //    vPhaseNum              : Phase number 1-8 to indicate Moon graphic
  //
  // return variables:
  var vRightAscensionDisplay ='';
  var vDeclinationDisplay    ='';
  var vElevationDisplay      ='';
  var vAzimuthDisplay        ='';
  var vDistanceDisplay       ='';
  var vPhaseAngleDisplay     ='';
  var vIlluminationDisplay   ='';
  var vMoonRiseDisplay       ='';
  var vMoonSetDisplay        ='';
  var vTimeDisplay           ='';
  var vPhaseNum              =15;

  if (argYear <1000 || argYear >3000) {
    //alert(msgOutOfSkyTimesRange);
    return ([vRightAscensionDisplay, vDeclinationDisplay, vElevationDisplay, vAzimuthDisplay,
           vDistanceDisplay, vPhaseAngleDisplay, vIlluminationDisplay,
           vMoonRiseDisplay, vMoonSetDisplay, vTimeDisplay, vPhaseNum]);
  }

  var vLatitude  = fGetLocation(argLatDeg, argLatMin, argLatSec, argLatDir);
  var vLongitude = fGetLocation(argLongDeg, argLongMin, argLongSec, argLongDir);
  if (vLatitude < -89.8 || vLatitude > 89.8) {
    alert(msgLatTooCloseToPoles);
    vLatitude = (argLatDir == 0 ? 89.8 : -89.8);
    with (argFrmLocation) {  //  Note call back to form
      frmLatDegInput.value = 89;
      frmLatMinInput.value = 48;
      frmLatSecInput.value = 0;
    }
  }
  // Calculate the Julian date
  var vJDay = fJDayFromYMD(argYear, argMonth, argDay);

  var vZone = Math.abs(argZone) + (+argZnAdj);
  vZone     = (argZone < 0 ? vZone : -vZone);  // reverse sign
  var vZoneAdjust = 60*vZone - argDST; // in minutes

  if (argTimeOverride) { // if set, use the time values that the user provided
    var vNowArray = fMinutesToHM(argHours*60.0 + (+argMinutes) - argDST);
    vNowArray.push(+argSeconds);  // add seconds as array element
  } else {
    var vNowArray = fGetTimeArray(vZoneAdjust + (+argDST));
  }

  var vDisplayArray = fMinutesToHMS (vNowArray[0]*60.0 + vNowArray[1] + vNowArray[2]/60.0 + (+argDST));
  var vTimeDisplay  = fFormatTime(vDisplayArray, vZoneAdjust, vJDay, argTimeStyle);

  var vMoonArray    = fMoonPosition(vLatitude,vLongitude,argYear,argMonth,argDay,vNowArray[0],vNowArray[1],vNowArray[2], vZoneAdjust+(+argDST));

  vRightAscensionDisplay = fHMS_String(vMoonArray[0]);
  vDeclinationDisplay    = fAngleString('DegMin', vMoonArray[1], false);
//   uncomment the next 2 lines, AND comment out the 2 lines after, to display Degrees & Minutes rather than decimal degrees
//   vElevationDisplay      = fAngleString('DegMin', vMoonArray[3], false);
//   vAzimuthDisplay        = fAngleString('DegMin', vMoonArray[4], true);
  vElevationDisplay      = fTruncateNPlaces(vMoonArray[3], 1) + msgDegrees;
  vAzimuthDisplay        = fTruncateNPlaces(vMoonArray[4], 1) + msgDegrees;
  vDistanceDisplay       = fCommaSplit(vMoonArray[2]) + ' km';
  var vPhaseAngle        = Math.round(vMoonArray[5]);
  vPhaseAngleDisplay     = vPhaseAngle + msgDegrees;
  vIlluminationDisplay   = Math.round(100.0*(1.0+fCosD(vPhaseAngle))/2.0) + '%';

  var vPhaseNum          = 1 + Math.floor(((vPhaseAngle+6.2069)/12.4138))%29; // range is 1-29; 6.2069 is added to put wedge into right 29-part sector
  // force certain angles to correspond to New, Quarter, and Full Moon images, thus ignoring rounding errors
  if (vPhaseAngle>354 && vPhaseAngle < 6)   vPhaseNum = 29;
  if (vPhaseAngle>84  && vPhaseAngle < 96)  vPhaseNum =  8;
  if (vPhaseAngle>174 && vPhaseAngle < 186) vPhaseNum = 15;
  if (vPhaseAngle>264 && vPhaseAngle < 276) vPhaseNum = 22;

  var vRiseSetArray = fMoonRiseSet(vLatitude, vLongitude, argYear, argMonth, argDay, vZoneAdjust);
  // -1 : means rise or set event not found and Moon was down at midnight
  // -2 : means rise or set event not found and Moon was up at midnight
  if ((vRiseSetArray[0] < 0) && (vRiseSetArray[1] < 0)) {
    switch (vRiseSetArray[0]) {
      case -1:
        vMoonRiseDisplay = msgNoRise + '; ' + msgMoonDown;
        vMoonSetDisplay  = msgNoSet  + '; ' + msgMoonDown;
        break;
      case -2:
        vMoonRiseDisplay = msgNoRise + '; ' + msgMoonUp;
        vMoonSetDisplay  = msgNoSet  + '; ' + msgMoonUp;
        break;
    }
  } else {
    if (vRiseSetArray[0] < 0) {
      vMoonRiseDisplay = msgNoRise;
    } else {
      var vMoonRise = 60*vRiseSetArray[0]; // convert vRiseSetArray to minutes
      switch (argTimeStyle) {
        case 0: vMoonRiseDisplay = fTimeStrSelector(vMoonRise, vJDay, 'AMPMDate'); break;
        case 1: vMoonRiseDisplay = fTimeStrSelector(vMoonRise, vJDay, '24HrDate'); break;
        case 2: vMoonRiseDisplay = msg_utc+' ' + fTimeStrSelector(vMoonRise - vZoneAdjust, vJDay, '24HrDate'); break;
      }
      var vNowArray    = fMinutesToHMS (vMoonRise);
      vMoonArray       = fMoonPosition(vLatitude,vLongitude,argYear,argMonth,argDay,vNowArray[0],vNowArray[1],vNowArray[2], vZoneAdjust);
      vMoonRiseDisplay = vMoonRiseDisplay + ' (' + Math.round(vMoonArray[4]) + msgDegrees + ')';
    }
    if (vRiseSetArray[1] < 0) {
      vMoonSetDisplay = msgNoSet;
    } else {
      var vMoonSet = 60*vRiseSetArray[1]; // convert vRiseSetArray to minutes
      switch (argTimeStyle) { 
        case 0: vMoonSetDisplay = fTimeStrSelector(vMoonSet, vJDay, 'AMPMDate'); break;
        case 1: vMoonSetDisplay = fTimeStrSelector(vMoonSet, vJDay, '24HrDate'); break;
        case 2: vMoonSetDisplay = msg_utc+' ' + fTimeStrSelector(vMoonSet - vZoneAdjust, vJDay, '24HrDate'); break;
      }
      var vNowArray   = fMinutesToHMS (vMoonSet);
      vMoonArray      = fMoonPosition(vLatitude,vLongitude,argYear,argMonth,argDay,vNowArray[0],vNowArray[1],vNowArray[2], vZoneAdjust);
      vMoonSetDisplay = vMoonSetDisplay + ' (' + Math.round(vMoonArray[4]) + msgDegrees + ')';
    }
  }

  return ([vRightAscensionDisplay, vDeclinationDisplay, vElevationDisplay, vAzimuthDisplay,
           vDistanceDisplay, vPhaseAngleDisplay, vIlluminationDisplay,
           vMoonRiseDisplay, vMoonSetDisplay, vTimeDisplay, vPhaseNum]);
}